<?php
/**
 * @version		$Id: weblinks.php 12490 2009-07-06 11:57:32Z eddieajau $
 * @package		Joomla
 * @copyright	Copyright (C) 2005 - 2009 Open Source Matters, Inc. All rights reserved.
 * @license		GNU General Public License version 2 or later; see LICENSE.txt
 */

// no direct access
defined('_JEXEC') or die;

$app = &JFactory::getApplication();
$app->registerEvent('onSearch', 'plgSearchWeblinks');
$app->registerEvent('onSearchAreas', 'plgSearchWeblinksAreas');

JPlugin::loadLanguage('plg_search_weblinks');

/**
 * @return array An array of search areas
 */
function &plgSearchWeblinksAreas() {
	static $areas = array(
		'weblinks' => 'Weblinks'
	);
	return $areas;
}

/**
* Weblink Search method
*
* The sql must return the following fields that are used in a common display
* routine: href, title, section, created, text, browsernav
* @param string Target search string
* @param string mathcing option, exact|any|all
* @param string ordering option, newest|oldest|popular|alpha|category
 * @param mixed An array if the search it to be restricted to areas, null if search all
 */
function plgSearchWeblinks($text, $phrase='', $ordering='', $areas=null)
{
	$db		= &JFactory::getDbo();
	$user	= &JFactory::getUser();
	$groups	= implode(',', $user->authorisedLevels());

	$searchText = $text;

	require_once(JPATH_SITE.DS.'components'.DS.'com_weblinks'.DS.'helpers'.DS.'route.php');

	if (is_array($areas)) {
		if (!array_intersect($areas, array_keys(plgSearchWeblinksAreas()))) {
			return array();
		}
	}

	// load plugin params info
 	$plugin = &JPluginHelper::getPlugin('search', 'weblinks');
 	$pluginParams = new JParameter($plugin->params);

	$limit = $pluginParams->def('search_limit', 50);

	$text = trim($text);
	if ($text == '') {
		return array();
	}
	$section 	= JText::_('Web Links');

	$wheres 	= array();
	switch ($phrase)
	{
		case 'exact':
			$text		= $db->Quote('%'.$db->getEscaped($text, true).'%', false);
			$wheres2 	= array();
			$wheres2[] 	= 'a.url LIKE '.$text;
			$wheres2[] 	= 'a.description LIKE '.$text;
			$wheres2[] 	= 'a.title LIKE '.$text;
			$where 		= '(' . implode(') OR (', $wheres2) . ')';
			break;

		case 'all':
		case 'any':
		default:
			$words 	= explode(' ', $text);
			$wheres = array();
			foreach ($words as $word)
			{
				$word		= $db->Quote('%'.$db->getEscaped($word, true).'%', false);
				$wheres2 	= array();
				$wheres2[] 	= 'a.url LIKE '.$word;
				$wheres2[] 	= 'a.description LIKE '.$word;
				$wheres2[] 	= 'a.title LIKE '.$word;
				$wheres[] 	= implode(' OR ', $wheres2);
			}
			$where 	= '(' . implode(($phrase == 'all' ? ') AND (' : ') OR ('), $wheres) . ')';
			break;
	}

	switch ($ordering)
	{
		case 'oldest':
			$order = 'a.date ASC';
			break;

		case 'popular':
			$order = 'a.hits DESC';
			break;

		case 'alpha':
			$order = 'a.title ASC';
			break;

		case 'category':
			$order = 'b.title ASC, a.title ASC';
			break;

		case 'newest':
		default:
			$order = 'a.date DESC';
	}

	$query = 'SELECT a.title AS title, a.description AS text, a.date AS created, a.url, '
	. ' CASE WHEN CHAR_LENGTH(a.alias) THEN CONCAT_WS(\':\', a.id, a.alias) ELSE a.id END as slug, '
	. ' CASE WHEN CHAR_LENGTH(b.alias) THEN CONCAT_WS(\':\', b.id, b.alias) ELSE b.id END as catslug, '
	. ' CONCAT_WS(" / ", '.$db->Quote($section).', b.title) AS section,'
	. ' "1" AS browsernav'
	. ' FROM #__weblinks AS a'
	. ' INNER JOIN #__categories AS b ON b.id = a.catid'
	. ' WHERE ('. $where .')'
	. ' AND a.published = 1'
	. ' AND b.published = 1'
	. ' AND b.access IN ('.$groups.')'
	. ' ORDER BY '. $order
	;
	$db->setQuery($query, 0, $limit);
	$rows = $db->loadObjectList();

	foreach($rows as $key => $row) {
		$rows[$key]->href = WeblinksHelperRoute::getWeblinkRoute($row->slug, $row->catslug);
	}

	$return = array();
	foreach($rows AS $key => $weblink) {
		if (searchHelper::checkNoHTML($weblink, $searchText, array('url', 'text', 'title'))) {
			$return[] = $weblink;
		}
	}

	return $return;
}
